## START: Set by rpmautospec
## (rpmautospec version 0.2.5)
%define autorelease(e:s:pb:) %{?-p:0.}%{lua:
    release_number = 6;
    base_release_number = tonumber(rpm.expand("%{?-b*}%{!?-b:1}"));
    print(release_number + base_release_number - 1);
}%{?-e:.%{-e*}}%{?-s:.%{-s*}}%{?dist}
## END: Set by rpmautospec

%global forgeurl https://github.com/biojppm/c4core

Name:           c4core
Summary:        C++ core utilities
Version:        0.1.6
# This is the same as the version number. To prevent undetected soversion
# bumps, we nevertheless express it separately.
%global so_version 0.1.6
%forgemeta
Release:        %autorelease

URL:            %{forgeurl}
# The entire source is MIT, except:
#
# Boost:
#   - src/c4/ext/sg14/inplace_function.h
License:        MIT and Boost

Source0:        %{forgesource}

# Disable “install” tests, which try to actually install files system-wide.
# This might be useful upstream (so the patch remains downstream-only), but it
# won’t be possible here.
Patch0:         c4core-0.1.7-no-install-tests.patch

BuildRequires:  gcc-c++
BuildRequires:  cmake
BuildRequires:  c4project
# Our choice; the default make backend should work just as well
BuildRequires:  ninja-build

# For each header-only library, the guidelines require us to BR the -static
# package for tracking.
BuildRequires:  debugbreak-devel
BuildRequires:  debugbreak-static
BuildRequires:  fast_float-devel
BuildRequires:  fast_float-static
BuildRequires:  doctest-devel
BuildRequires:  doctest-static

%global common_description %{expand:
c4core is a library of low-level C++ utilities, written with low-latency
projects in mind.

Some of the utilities provided by c4core have already equivalent functionality
in the C++ standard, but they are provided as the existing C++ equivalent may
be insufficient (eg, std::string_view), inefficient (eg, std::string), heavy
(eg streams), or plainly unusable on some platforms/projects, (eg exceptions);
some other utilities have equivalent under consideration for C++
standardization; and yet some other utilities have (to my knowledge) no
equivalent under consideration.}

%description %{common_description}


%package devel
Summary:        %{summary}

Requires:       %{name}%{?_isa} = %{version}-%{release}

# Each of these header-only libraries is made available under c4/ext/… in the
# API of this package. Dependent packages that use them should really have
# BuildRequires on the corresponding -static packages for header-only package
# tracking.
Requires:       debugbreak-devel
Requires:       fast_float-devel

# The bundled copy was forked from the original header-only library published
# as a gist at
# https://gist.github.com/Leandros/6dc334c22db135b033b57e9ee0311553 (see also
# the blog post at https://arvid.io/2018/07/02/better-cxx-prng/). The original
# was never versioned, but only one revision was ever published, so we can
# infer the commit hash (6dc334c22db135b033b57e9ee0311553) from which the
# bundled copy was forked and assign a snapshot version.
Provides:       bundled(ag-random) = 0^20180702git6dc334c
# One header is bundled from https://github.com/WG21-SG14/SG14/ (commit
# 3aeb80676ff3e7a974678bd3fd826ffe55a0c4ab), which has never been versioned and
# is not currently suitable to be packaged in its entirety—if nothing else, due
# to unresolved licensing in some of its other source files that are not
# bundled here, https://github.com/WG21-SG14/SG14/issues/163.
Provides:       bundled(SG14) = 0^20190524git3a3b806

%description devel %{common_description}


%prep
%forgeautosetup -p1

# Remove/unbundle additional dependencies

# c4project (CMake build scripts)
rm -rvf cmake
cp -rvp %{_datadir}/cmake/c4project cmake
# Patch out download of doctest:
awk '/^    if\(_DOCTEST\)$/ { x=1 }
     x != 1
     x == 1 && /^    endif\(\)$/ { x=0 }' \
    '%{_datadir}/cmake/c4project/c4Project.cmake' > cmake/c4Project.cmake
# Do not try to link against a nonexistent doctest library (doctest is
# header-only, and we do not have the complete CMake project for doctest that
# would provide a target that knows this):
sed -r -i 's/\bdoctest\b//' test/CMakeLists.txt

# debugbreak
rm -rvf src/c4/ext/debugbreak
mkdir src/c4/ext/debugbreak
ln -sv %{_includedir}/debugbreak.h src/c4/ext/debugbreak/

# fast_float
#
# The wrapper header src/c4/ext/fast_float.hpp includes fast_float.h using a
# relative path, so we must use a symbolic link to preserve the original
# bundled header path.
rm -rvf src/c4/ext/fast_float
mkdir -p src/c4/ext/fast_float/include
ln -sv %{_includedir}/fast_float src/c4/ext/fast_float/include/
# F33-F34 have an older fast_float (1.1.2 instead of 3.x), which lacks a couple
# of headers listed as sources:
sed -r -i '/\/fast_float\/(bigint|digit_comparison)\.h$/d' CMakeLists.txt


%build
%cmake -DC4CORE_BUILD_TESTS=ON -GNinja
%cmake_build


%install
%cmake_install
# Fix wrong installation paths for multilib; it would be nontrivial to patch
# the source to get this right in the first place.
if [ '%{_libdir}' != '%{_prefix}/lib' ]
then
  mkdir -p '%{buildroot}%{_libdir}'
  mv -v %{buildroot}%{_prefix}/lib/libc4core.so* '%{buildroot}%{_libdir}/'
  mkdir -p '%{buildroot}%{_libdir}/cmake'
  mv -v %{buildroot}%{_prefix}/lib/cmake/c4core '%{buildroot}%{_libdir}/cmake/'
fi

# Some unbundled header-only libraries that appear in the API may have had
# their symlinks dereferenced during installation. Make sure they aren’t
# “re-bundled” as a result.

# debugbreak
ln -svf '%{_includedir}/debugbreak.h' \
    '%{buildroot}%{_includedir}/c4/ext/debugbreak/'

# fast_float
rm -rvf '%{buildroot}%{_includedir}/c4/ext/fast_float/include/fast_float'
ln -svf '%{_includedir}/fast_float' \
    '%{buildroot}%{_includedir}/c4/ext/fast_float/include/'


%check
%cmake_build --target c4core-test-run-verbose


%files
%license LICENSE.txt LICENSE-BOOST.txt
%doc README.md
%doc ROADMAP.md
%doc changelog/
%{_libdir}/libc4core.so.%{so_version}


%files devel
%{_includedir}/c4
%{_libdir}/libc4core.so
%{_libdir}/cmake/c4core


%changelog
* Fri Dec 17 2021 Benjamin A. Beasley <code@musicinmybrain.net> 0.1.6-6
- De-arch two -devel dependencies

* Fri Dec 17 2021 Benjamin A. Beasley <code@musicinmybrain.net> 0.1.6-5
- Dep. on cmake-filesystem is now auto-generated

* Fri Dec 17 2021 Benjamin A. Beasley <code@musicinmybrain.net> 0.1.6-4
- Remove an unnecessary glob in the files list

* Fri Dec 17 2021 Benjamin A. Beasley <code@musicinmybrain.net> 0.1.6-3
- Build and run tests a simpler way

* Tue Oct 26 2021 Benjamin A. Beasley <code@musicinmybrain.net> 0.1.6-2
- Work around older fast_float 1.x in F33–F34

* Tue Oct 26 2021 Benjamin A. Beasley <code@musicinmybrain.net> 0.1.6-1
- Initial package
